#include <stdio.h>
#include <crypto_auth.h>
#include <crypto_auth_hmacsha256.h>
#include <crypto_auth_hmacsha512256.h>
#include <crypto_box_curve25519xsalsa20poly1305.h>
#include <crypto_box.h>
#include <crypto_core_hsalsa20.h>
#include <crypto_core_salsa2012.h>
#include <crypto_core_salsa208.h>
#include <crypto_core_salsa20.h>
#include <crypto_hashblocks.h>
#include <crypto_hashblocks_sha256.h>
#include <crypto_hashblocks_sha512.h>
#include <crypto_hash.h>
#include <crypto_hash_sha256.h>
#include <crypto_hash_sha512.h>
#include <crypto_int16.h>
#include <crypto_int8.h>
#include <crypto_onetimeauth.h>
#include <crypto_onetimeauth_poly1305.h>
#include <crypto_scalarmult_curve25519.h>
#include <crypto_secretbox.h>
#include <crypto_secretbox_xsalsa20poly1305.h>
#include <crypto_sign_edwards25519sha512batch.h>
#include <crypto_sign.h>
#include <crypto_stream_aes128ctr.h>
#include <crypto_stream.h>
#include <crypto_stream_salsa2012.h>
#include <crypto_stream_salsa208.h>
#include <crypto_stream_salsa20.h>
#include <crypto_stream_xsalsa20.h>
#include <crypto_uint16.h>
#include <crypto_uint32.h>
#include <crypto_uint64.h>
#include <crypto_uint8.h>
#include <crypto_verify_16.h>
#include <crypto_verify_32.h>
#include <randombytes.h>

#define ZEROBYTES crypto_box_ZEROBYTES

static unsigned char alicesk[crypto_box_SECRETKEYBYTES];
static unsigned char alicepk[crypto_box_PUBLICKEYBYTES];
static unsigned char bobsk[crypto_box_SECRETKEYBYTES];
static unsigned char bobpk[crypto_box_PUBLICKEYBYTES];
static unsigned char n[crypto_box_NONCEBYTES + 16];
static unsigned char k[32 + 16];
static unsigned char a[16 + 16];
#define SPACE 10001
static unsigned char m[SPACE + 16];
static unsigned char c[SPACE + 16];
static unsigned char m2[SPACE + 16];

int main(void) {
    long long mlen, i;

    /* test box */
    for (mlen = 0; mlen < SPACE/10 && mlen + ZEROBYTES < SPACE; ++mlen) {
        crypto_box_keypair(alicepk, alicesk);
        crypto_box_keypair(bobpk, bobsk);
        randombytes(n, crypto_box_NONCEBYTES);
        randombytes(m + ZEROBYTES, mlen);
        crypto_box(c, m, mlen + ZEROBYTES, n, bobpk, alicesk);
        if (crypto_box_open(m2, c, mlen + ZEROBYTES, n, alicepk, bobsk) == 0) {
            for (i = 0; i < mlen + ZEROBYTES; ++i) {
                if (m2[i] != m[i]) {
                    printf("bad decryption\n");
                    return 1;
                }
            }
        }
        else {
            printf("ciphertext fails verification\n");
            return 1;
        }
    }

    /* test stream + onetimeauth */
    for (i = 0; i < 16; ++i) {
        randombytes(k, sizeof k);
        randombytes(n, sizeof n);
        crypto_stream_aes128ctr(c + i, SPACE, n + i, k + i);
        randombytes(k, sizeof k);
        randombytes(n, sizeof n);
        crypto_stream_salsa20_xor(c + i, c + i, SPACE, n + i, k + i);
        randombytes(k, sizeof k);
        randombytes(n, sizeof n);
        crypto_stream_salsa2012_xor(c + i, c + i, SPACE, n + i, k + i);
        randombytes(k, sizeof k);
        randombytes(n, sizeof n);
        crypto_stream_salsa208_xor(c + i, c + i, SPACE, n + i, k + i);
        randombytes(k, sizeof k);
        crypto_onetimeauth_poly1305(a + i, c + i, SPACE, k + i);
        if (crypto_onetimeauth_poly1305_verify(a + i, c + i, SPACE, k + i) != 0) {
            printf("ciphertext fails verification\n");
            return 1;
        }

    }

    /* print implementation */
    printf("%s\n", crypto_auth_IMPLEMENTATION);
    printf("%s\n", crypto_auth_hmacsha256_IMPLEMENTATION);
    printf("%s\n", crypto_auth_hmacsha512256_IMPLEMENTATION);
    printf("%s\n", crypto_box_curve25519xsalsa20poly1305_IMPLEMENTATION);
    printf("%s\n", crypto_box_IMPLEMENTATION);
    printf("%s\n", crypto_core_hsalsa20_IMPLEMENTATION);
    printf("%s\n", crypto_core_salsa2012_IMPLEMENTATION);
    printf("%s\n", crypto_core_salsa208_IMPLEMENTATION);
    printf("%s\n", crypto_core_salsa20_IMPLEMENTATION);
    printf("%s\n", crypto_hashblocks_IMPLEMENTATION);
    printf("%s\n", crypto_hashblocks_sha256_IMPLEMENTATION);
    printf("%s\n", crypto_hashblocks_sha512_IMPLEMENTATION);
    printf("%s\n", crypto_hash_IMPLEMENTATION);
    printf("%s\n", crypto_hash_sha256_IMPLEMENTATION);
    printf("%s\n", crypto_hash_sha512_IMPLEMENTATION);
    printf("%s\n", crypto_onetimeauth_IMPLEMENTATION);
    printf("%s\n", crypto_onetimeauth_poly1305_IMPLEMENTATION);
    printf("%s\n", crypto_scalarmult_curve25519_IMPLEMENTATION);
    printf("%s\n", crypto_secretbox_IMPLEMENTATION);
    printf("%s\n", crypto_secretbox_xsalsa20poly1305_IMPLEMENTATION);
    printf("%s\n", crypto_sign_edwards25519sha512batch_IMPLEMENTATION);
    printf("%s\n", crypto_sign_IMPLEMENTATION);
    printf("%s\n", crypto_stream_aes128ctr_IMPLEMENTATION);
    printf("%s\n", crypto_stream_IMPLEMENTATION);
    printf("%s\n", crypto_stream_salsa2012_IMPLEMENTATION);
    printf("%s\n", crypto_stream_salsa208_IMPLEMENTATION);
    printf("%s\n", crypto_stream_salsa20_IMPLEMENTATION);
    printf("%s\n", crypto_stream_xsalsa20_IMPLEMENTATION);
    printf("%s\n", crypto_verify_16_IMPLEMENTATION);
    printf("%s\n", crypto_verify_32_IMPLEMENTATION);
    return 0;
}
